import subprocess

from tchotchke.exceptions import ShellError


class Shell:
    def __init__(self, logger):
        self.__logger = logger

    def execute_for_stdout(self, command, insured=True):
        process_result = self.execute(command, insured)
        return process_result.stdout.strip()

    def execute(self, command, insured=True, capture_output=True):
        process_result = subprocess.run(command, capture_output=capture_output, shell=True, encoding="UTF-8")
        if insured:
            self.__ensure_successful_return_code(process_result)
        return process_result

    @staticmethod
    def __ensure_successful_return_code(process_result):
        if process_result.returncode != 0:
            raise ShellError("subprocess command failed", {"process_result": repr(process_result)})



import sys
import traceback
from contextlib import contextmanager

import loguru
import ujson
from tchotchke.exceptions import LoggableError


class Logger:
    def __init__(self, log_sink=sys.stdout):
        self.__internal_logger = loguru.logger
        self.__internal_logger.remove()
        self.__internal_logger.add(log_sink, backtrace=True, diagnose=True, colorize=True)

    @contextmanager
    def log_uncaught_exceptions(self):
        try:
            yield
        except Exception as error:
            self.exception("an unhandled exception occurred", error)
            raise

    def debug(self, message, sprinkles=None):
        output_log = self.__format_output_log(message, sprinkles)
        serialized_output = ujson.dumps(output_log)
        self.__internal_logger.debug(serialized_output)

    def info(self, message, sprinkles=None):
        output_log = self.__format_output_log(message, sprinkles)
        serialized_output = ujson.dumps(output_log)
        self.__internal_logger.info(serialized_output)

    def warning(self, message, sprinkles=None):
        output_log = self.__format_output_log(message, sprinkles)
        serialized_output = ujson.dumps(output_log)
        self.__internal_logger.warning(serialized_output)

    def error(self, message, sprinkles=None):
        output_log = self.__format_output_log(message, sprinkles)
        serialized_output = ujson.dumps(output_log)
        self.__internal_logger.error(serialized_output)

    def exception(self, message, error, sprinkles=None):
        if sprinkles is None:
            sprinkles = {}
        if isinstance(error, LoggableError):
            sprinkles |= error.log_sprinkles
        output_log = self.__format_output_log(message, sprinkles)
        output_log["exception"] = traceback.format_exc()
        output_log["error_message"] = str(error)
        serialized_output = ujson.dumps(output_log)
        self.__internal_logger.error(serialized_output)

    @staticmethod
    def __format_output_log(message, raw_sprinkles):
        if not isinstance(message, str):
            raise TypeError()
        if not isinstance(raw_sprinkles, dict) and raw_sprinkles is not None:
            raise TypeError()
        sprinkles = Logger.route_recurse(raw_sprinkles)
        return {
            "message": message,
            "sprinkles": sprinkles
        }

    @staticmethod
    def route_recurse(value):
        if isinstance(value, list):
            return Logger.__repr_list(value)
        elif isinstance(value, dict):
            return Logger.__repr_dict(value)
        else:
            return repr(value)

    @staticmethod
    def __repr_list(sprinkles):
        result = []
        for value in sprinkles:
            formatted_value = Logger.route_recurse(value)
            result.append(formatted_value)
        return result

    @staticmethod
    def __repr_dict(sprinkles):
        result = {}
        for key, value in sprinkles.items():
            formatted_value = Logger.route_recurse(value)
            result[key] = formatted_value
        return result



class TchotchkeError(Exception):
    pass


class LoggableError(TchotchkeError):
    def __init__(self, message=None, log_sprinkles=None):
        super().__init__(message)
        if log_sprinkles is None:
            log_sprinkles = {}
        self.log_sprinkles = log_sprinkles


class ConfigError(LoggableError):
    pass


class ShellError(LoggableError):
    pass



import os

import yaml

from tchotchke.exceptions import ConfigError


class MetaConfig(type):
    CONFIG_PATH_ENVIRONMENT_VARIABLE = "CONFIG_FILE_PATH"

    def __new__(cls, name, bases, dct):
        config = super().__new__(cls, name, bases, dct)
        config_path = os.getenv(MetaConfig.CONFIG_PATH_ENVIRONMENT_VARIABLE)
        if config_path is None:
            raise ConfigError(f"environment variable: {MetaConfig.CONFIG_PATH_ENVIRONMENT_VARIABLE} not set")
        try:
            with open(config_path, "r") as input_file:
                yaml_config = yaml.safe_load(input_file)
        except FileNotFoundError:
            raise ConfigError(f"failed to locate config file at: {config_path}")
        if yaml_config is None:
            raise ConfigError("failed to parse config yaml")
        for key, value in yaml_config.items():
            setattr(config, key, value)
        return config

    def __getattr__(self, attribute_name):
        raise ConfigError(f"unknown config attribute: {attribute_name}, ensure it exists in yaml config")


class Config(metaclass=MetaConfig):
    pass



from setuptools import setup

installation_requirements = [
    "loguru==0.5.3",
    "ujson==4.0.2",
    "pyyaml==5.4.1"
]

setup(
    name="tchotchke",
    description="Trinkets and baubles to adorn your pythonic pursuits.",
    version="0.1.16",
    url="https://github.com/josiahdc/tchotchke",
    author="Josiah Chapman",
    author_email="josiah.chapman@gmail.com",
    package_dir={"": "src"},
    packages=["tchotchke"],
    install_requires=installation_requirements
)
